#include "launcher.h"

#include <QDebug>
#include <QObject>
#include <QWidget>
#include <QDir>
#include <QCoreApplication>
#include <QString>
#include <QList>
#include <qjsonobject.h>
#include <QEventLoop>

#include "profile_handler/forge1_12_2.h"
#include "../ui/mainwindow.h"
#include "../../resources/ui_mainwindow.h"


Launcher::Launcher(QObject *mainwindow) : QObject(mainwindow), process(new QProcess(this)), mainwindow(mainwindow), get_args(new forge1_12_2()), run_status(false) {
}


Launcher::~Launcher() {
    delete process;
}


void Launcher::update() {
    MainWindow *mainWindow = qobject_cast<MainWindow *>(mainwindow);
    QDir app_dir(QCoreApplication::applicationDirPath());

    // Создаем процесс для выполнения команды rsync
    QProcess *rsyncProcess = new QProcess(this);
    rsyncProcess->setWorkingDirectory(app_dir.absolutePath());


    // Подключаем вывод процесса к текстовому полю download_bar
    connect(rsyncProcess, &QProcess::readyReadStandardOutput, [rsyncProcess, mainWindow]() {
        QString output = rsyncProcess->readAllStandardOutput();
		QStringList lines = output.split('\r', Qt::SkipEmptyParts);
		QString lastLine = lines.isEmpty() ? "" : lines.last().trimmed();
		if (!lastLine.isEmpty()) {
			QRegularExpression regex(R"((\d+,\d+,\d+)\s+(\d+%)\s+(\d+\.\d+MB\/s)\s+(\d+:\d+:\d+))");
			QRegularExpressionMatch match = regex.match(lastLine);
			if (match.hasMatch()) {
				QString progress = match.captured(1);
				QString percent = match.captured(2);
				QString speed = match.captured(3);
				QString time = match.captured(4);
				QString displayText = QString("%1  %2  %3  %4").arg(progress, percent, speed, time);
				mainWindow->ui->download_bar->setText(displayText);
			}
		}
    });

    // Подключаем вывод stderr к текстовому полю download_bar
    connect(rsyncProcess, &QProcess::readyReadStandardError, [rsyncProcess, mainWindow]() {
        QString errorOutput = rsyncProcess->readAllStandardError();
		mainWindow->ui->download_bar->setText(errorOutput);
		qDebug() << errorOutput;
    });

    // Выполняем команду rsync
    QStringList rsyncArgs;
    rsyncArgs << "-az" << "--info=progress2" << "punkcraft.ru::PunkCraft" << "./";
    rsyncProcess->start("rsync", rsyncArgs);

    if (!rsyncProcess->waitForStarted()) {
        qDebug() << "Failed to start rsync process";
        qDebug() << rsyncProcess->errorString();
        emit updateFinished();  // Сигнализируем о завершении (даже если произошла ошибка)
        return;
    }

    // Ожидаем завершения процесса rsync
    connect(rsyncProcess, QOverload<int, QProcess::ExitStatus>::of(&QProcess::finished), [this, rsyncProcess, mainWindow](int, QProcess::ExitStatus) {
        qDebug() << "Rsync process finished";
        rsyncProcess->deleteLater();  // Удаляем процесс после завершения
        emit updateFinished();  // Сигнализируем о завершении
		mainWindow->ui->download_bar->setText("");
    });
}


void Launcher::startProcess(QString &username, QString &uuid, QString &access_token) {
    if (run_status) {
        qDebug() << "Process is already running";
        return;
    }
    run_status = true;

    MainWindow *mainWindow = qobject_cast<MainWindow *>(mainwindow);
    QDir app_dir(QCoreApplication::applicationDirPath());

    // Обновление клиента
    QEventLoop loop;
    connect(this, &Launcher::updateFinished, &loop, &QEventLoop::quit);  // Ожидаем сигнала updateFinished
    update();  // Запускаем асинхронное обновление
    loop.exec();  // Блокируем выполнение до завершения update

    // Подготовка параметров запуска
    QString selected_profile = mainWindow->ui->selected_profile->currentText();
    client_dir = app_dir.absolutePath() + "/" + selected_profile;
    json_settings = mainWindow->settings->loadFromJson(selected_profile);
    java_path = json_settings["java_path"].toString();
    RAM = json_settings["RAM"].toString();
    garbarge_collector = json_settings["garbarge_collector"].toString();

    //
    // Выбор профиля
    //
    if (selected_profile == "ZombieExtrieme") {
        arguments = get_args->getCrossPlatformArgs(RAM, username, uuid, access_token, garbarge_collector);
        java_path = java_path.isEmpty() ? 
            app_dir.absolutePath() + "/java/zulu8.84.0.15-ca-jre8.0.442-linux_x64/bin/java" : java_path;
        RAM = RAM.isEmpty() ? "4096M" : RAM;
        garbarge_collector = garbarge_collector.isEmpty() ? "default_collector" : garbarge_collector;
    }

    // Запуск процесса
    process->setWorkingDirectory(client_dir);
    process->start(java_path, arguments);

    if (!process->waitForStarted()) {
        qDebug() << "Failed to start process";
        qDebug() << process->errorString();
        run_status = false; // Сбрасываем статус, если процесс не запустился
        return;
    }

    // Скрытие лаунчера
    connect(process,
        QOverload<int, QProcess::ExitStatus>::of(&QProcess::finished),
        [mainWindow, this](int, QProcess::ExitStatus) { 
            mainWindow->show(); 
            run_status = false; // Сбрасываем статус после завершения процесса
        });
    mainWindow->hide();
    qDebug() << "Process started successfully";
}
